//  Sam's Particle Engine
//  Sam Christiansen
//  schristi@mail.utexas.edu

#include <windows.h>
#include <windowsx.h>
#include <ddraw.h>
#include "main.h"
#include "particle.h"


HWND				g_hwnd;
LPDIRECTDRAW		g_lpDD;		// DirectDraw object
LPDIRECTDRAWSURFACE	g_lpDDSPrimary;	// DirectDraw primary surface
LPDIRECTDRAWSURFACE	g_lpDDSBack;	// DirectDraw back surface
BOOL				bActive;		// is application active?





HRESULT restoreAll( void );

void finiObjects();

void fatalError(LPSTR mbMsg);


// updateFrame
void updateFrame( void )
{
	HRESULT ddrval;


	static ParticleSystem PS1, PS2, PS3;
	static int i=0, j=25, k=50;

	PS1.Calculate();
	PS1.Render(TRUE);

	PS2.Calculate();
	PS2.Render();

	PS3.Calculate();
	PS3.Render();

	i++;
	j++;
	k++;
	if (i > 75)
	{
		PS1.Reset();
		i=0;
	}
	if (j > 75)
	{
		PS2.Reset();
		j=0;
	}
	if (k > 75)
	{
		PS3.Reset();
		k=0;
	}
		



    // Flip the surfaces
    while( 1 )
    {
	ddrval = g_lpDDSPrimary->Flip( NULL, 0 );
	if( ddrval == DD_OK )
	{
	    break;
	}
	if( ddrval == DDERR_SURFACELOST )
	{
	    ddrval = restoreAll();
	    if( ddrval != DD_OK )
	    {
		break;
	    }
	}
	if( ddrval != DDERR_WASSTILLDRAWING )
	{
	    break;
	}
    }


} // updateFrame


//  our error handler routine
void fatalError(LPSTR mbMsg)
{
	finiObjects();
	if(mbMsg == NULL)
	{
		mbMsg = new char[64];
		wsprintf(mbMsg, "General error...");
	}
    MessageBox( g_hwnd, mbMsg, TITLE, MB_OK );
    DestroyWindow( g_hwnd );
	exit(1);
}  // fatalError



// restoreAll
// restore all lost objects
HRESULT restoreAll( void )
{
    HRESULT	ddrval;

    ddrval = g_lpDDSPrimary->Restore();
    if( ddrval == DD_OK )
    {

    }
    return ddrval;

} // restoreAll


// finiObjects
 
// finished with all objects we use; release them

static void finiObjects( void )
{
    if( g_lpDD != NULL )
    {
	if( g_lpDDSPrimary != NULL )
	{
	    g_lpDDSPrimary->Release();
	    g_lpDDSPrimary = NULL;
	}
	g_lpDD->Release();
	g_lpDD = NULL;
    }
} // finiObjects

long FAR PASCAL WindowProc( HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam )
{
    switch( message )
    {
    case WM_ACTIVATEAPP:
    	bActive = wParam;
	break;

    case WM_SETCURSOR:
        SetCursor(NULL);
        return TRUE;

    case WM_CREATE:
	break;

    case WM_KEYDOWN:
	switch( wParam )
	{
	case VK_ESCAPE:
        case VK_F12:
            PostMessage(hWnd, WM_CLOSE, 0, 0);
	    break;
	}
	break;

    case WM_DESTROY:
    	finiObjects();
	PostQuitMessage( 0 );
	break;
    }

    return DefWindowProc(hWnd, message, wParam, lParam);

} // WindowProc


// This function is called if the initialization function fails

BOOL initFail(  )
{
    finiObjects();
    MessageBox( g_hwnd, "DirectDraw Init FAILED", TITLE, MB_OK );
    DestroyWindow( g_hwnd );
    return FALSE;

} // initFail


// doInit - do work required for every instance of the application:
//		  create the window, initialize data

static BOOL doInit( HINSTANCE hInstance, int nCmdShow )
{
    WNDCLASS		wc;
    DDSURFACEDESC	ddsd;
    DDSCAPS		ddscaps;
    HRESULT		ddrval;

    
    // set up and register window class
    
    wc.style = CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc = WindowProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0;
    wc.hInstance = hInstance;
    wc.hIcon = LoadIcon( hInstance, IDI_APPLICATION );
    wc.hCursor = LoadCursor( NULL, IDC_ARROW );
    wc.hbrBackground = GetStockBrush(BLACK_BRUSH);
    wc.lpszMenuName = NAME;
    wc.lpszClassName = NAME;
    RegisterClass( &wc );
    
    
    // create a window
    
    g_hwnd = CreateWindowEx(
        0,
	NAME,
	TITLE,
	WS_POPUP,
	0,
	0,
        GetSystemMetrics(SM_CXSCREEN),
        GetSystemMetrics(SM_CYSCREEN),
	NULL,
	NULL,
	hInstance,
	NULL );

    if( !g_hwnd )
    {
	return FALSE;
    }

    ShowWindow( g_hwnd, nCmdShow );
    UpdateWindow( g_hwnd );

    
    // create the main DirectDraw object
    
    ddrval = DirectDrawCreate( NULL, &g_lpDD, NULL );
    if( ddrval != DD_OK )
    {
	return initFail();
    }

    // Get exclusive mode
    ddrval = g_lpDD->SetCooperativeLevel( g_hwnd, DDSCL_EXCLUSIVE | DDSCL_FULLSCREEN );
    if( ddrval != DD_OK )
    {
	return initFail();
    }

    // Set the video mode to 640x480x16
    ddrval = g_lpDD->SetDisplayMode( 640, 480, 16);
    if( ddrval != DD_OK )
    {
	return initFail();
    }

    // Create the primary surface with 1 back buffer
    ddsd.dwSize = sizeof( ddsd );
    ddsd.dwFlags = DDSD_CAPS | DDSD_BACKBUFFERCOUNT;
    ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE |
			  DDSCAPS_FLIP |
			  DDSCAPS_COMPLEX;
    ddsd.dwBackBufferCount = 1;
    ddrval = g_lpDD->CreateSurface( &ddsd, &g_lpDDSPrimary, NULL );
    if( ddrval != DD_OK )
    {
	return initFail();
    }

    ddscaps.dwCaps = DDSCAPS_BACKBUFFER;
    ddrval = g_lpDDSPrimary->GetAttachedSurface(&ddscaps, &g_lpDDSBack);
    if( ddrval != DD_OK )
    {
	return initFail();
    }

    return TRUE;
} // doInit


// WinMain - initialization, message loop

int PASCAL WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
    MSG		msg;

    lpCmdLine = lpCmdLine;
    hPrevInstance = hPrevInstance;

    if( !doInit( hInstance, nCmdShow ) )
    {
	return FALSE;
    }

    while( 1 )
    {
	if( PeekMessage( &msg, NULL, 0, 0, PM_NOREMOVE ) )
	{
	    if( !GetMessage( &msg, NULL, 0, 0 ) )
		return msg.wParam;
	    TranslateMessage(&msg); 
	    DispatchMessage(&msg);
	}
	else if( bActive )
	{
	    updateFrame();
        }
        else
        {
            // make sure we go to sleep if we have nothing else to do
            WaitMessage();
        }
    }
} // WinMain
