//  particle.cpp
//  particle engine implementation file
//  Sam Christiansen

#include <stdlib.h>
#include <time.h>
#include <math.h>
#include "particle.h"


// sets up new particle system with random vectors
ParticleSystem::ParticleSystem(int _size, int power, int energy)
{
	threshold_energy = energy;
	size = _size;
	particle_array = new Particle[size];

	if (!GetRGB16(g_lpDDSBack, &rgb16)) fatalError("GetRGB16 Failed.");


    ddsd.dwSize = sizeof( ddsd );
    ddsd.dwFlags = DDSD_PIXELFORMAT;
    if (g_lpDDSBack->GetSurfaceDesc ( &ddsd ) != DD_OK) 
		fatalError("GetSurfaceDesc, Particle.cpp failed!");



	double halfpower = power / 2;
	srand( (unsigned)time( NULL ) );

	for (int j=0;j<360;j++)   //  set up sine/cosine lookup tables
	{
		sine[j] = sin(j*3.1415926535897932/180);
		cosine[j] = cos(j*3.1415926535897932/180);
	}

	int x, y;   // explosion start position
	x = (rand() % 560) + 40;
	y = (rand() % 400) + 40;
	double speed=0;

	for (int i=0;i<size;i++)
	{
		particle_array[i].x			= x;
		particle_array[i].y			= y;
		particle_array[i].energy	= (rand() % MAX_ENERGY);
		particle_array[i].angle		= (rand() % 360);
		speed						= (rand() % power) - halfpower;
		particle_array[i].speedx	= sine[particle_array[i].angle] * speed;
		particle_array[i].speedy	= cosine[particle_array[i].angle] * speed;
		particle_array[i].r			= rand() % 31;
		particle_array[i].g			= 0;
		particle_array[i].b			= rand() % 31;
	}
}


ParticleSystem::~ParticleSystem()
{
	delete [] particle_array;
}

// calculate next frame according to speed/direction/energy
void ParticleSystem::Calculate()
{
	for (int i=0;i<size;i++)
	{
		if (particle_array[i].energy > threshold_energy)
		{
		particle_array[i].x			= particle_array[i].x + particle_array[i].speedx;
		if (particle_array[i].x > 640) { particle_array[i].speedx *= -0.5; particle_array[i].x = 639; }
		if (particle_array[i].x < 0) {particle_array[i].speedx *= -0.5; particle_array[i].x = 0; }
		particle_array[i].y			= particle_array[i].y + particle_array[i].speedy;
		if (particle_array[i].y > 480) { particle_array[i].speedy *= -0.3; particle_array[i].y = 479; }
		if (particle_array[i].y < 0) {particle_array[i].speedy *= -1; particle_array[i].y = 0; }
		particle_array[i].energy	-= 1;
//		particle_array[i].speedx	= sine[particle_array[i].angle] * speed;
		particle_array[i].speedy	+= 0.2;

		}
	}
}





// render particle system
void ParticleSystem::Render(BOOL clear)
{
	//  lock the surface
	Lock( &ddsd );

	//  clear our buffer
	if (clear)
		memset (ddsd.lpSurface, 0, 614400);  //64*480*2
	for (int i=0;i<size;i++)
		if (particle_array[i].energy > threshold_energy)
		{
			PlotPixel((int)particle_array[i].x,
				      (int)particle_array[i].y, 
					  particle_array[i].r, 
					  particle_array[i].g, 
					  particle_array[i].b, 
					  &ddsd, 
					  &rgb16);
		}

	//  unlock the surface
	UnLock();
}



void ParticleSystem::Reset(int _size, int power, int energy)
{
	threshold_energy = energy;
	size = _size;
	delete [] particle_array;
	particle_array = new Particle[size];

	int halfpower = power / 2;

	int x, y;   // explosion start position
	x = (rand() % 560) + 40;
	y = (rand() % 400) + 40;
	double speed=0;

	for (int i=0;i<size;i++)
	{
		particle_array[i].x			= x;
		particle_array[i].y			= y;
		particle_array[i].energy	= (rand() % MAX_ENERGY);
		particle_array[i].angle		= (rand() % 360);
		speed						= (rand() % power) - halfpower;
		particle_array[i].speedx	= sine[particle_array[i].angle] * speed;
		particle_array[i].speedy	= cosine[particle_array[i].angle] * speed;
		particle_array[i].r			= rand() % 31;
		particle_array[i].g			= 0;
		particle_array[i].b			= rand() % 31;
	}

}