//  FILE:  Particle.h
//  header file for my particle engine (2d)
//  Sam Christiansen

#ifndef _PARTICLE_H
#define _PARTICLE_H

#define MAX_ENERGY 60
#define DEFAULT_SIZE 200
#define DEFAULT_POWER 30
#define DEFAULT_ENERGY 0

#include <ddraw.h>	//  directdraw stuff	
#include "pixel.h"	//  pixel routines
#include "main.h"

typedef struct {
	//position:
	double x,
		y;
	//speed
	double speedx,
		speedy;
	//energy
	int energy;
	//direction
	int angle;
	//color
	int r,
		g,
		b;
} Particle;

class ParticleSystem {
	public:
		// sets up new particle system with random vectors
		ParticleSystem(int _size=DEFAULT_SIZE, int power=DEFAULT_POWER, int energy=DEFAULT_ENERGY);
		~ParticleSystem();
		// calculate next frame according to speed/direction/energy
		void Calculate();
		// render particle system
		void Render(BOOL clear=FALSE);
		// reset particle system to random values
		void Reset(int _size=DEFAULT_SIZE, int power=DEFAULT_POWER, int energy=DEFAULT_ENERGY);
	private:
		//  array holding n numbers of Particles
		Particle * particle_array;
		//  threshold energy before particle is considered dead
		int threshold_energy;
		//  size:
		int size;
		//  pixel stuff:
		RGB16 rgb16;
		//  direct draw stuff:
		DDSURFACEDESC ddsd;
		//  cosine/sine lookup tables
		double sine[360], cosine[360];
};


#endif